@@ -23,7 +23,7 @@ from logging import getLogger
 from threading import Lock
 from time import strptime
 from types import TracebackType
-from typing import Any, Callable, Generator, Iterable, NamedTuple, Sequence
+from typing import Any, Callable, Generator, Iterable, Iterator, NamedTuple, Sequence
 from uuid import UUID
 
 from cryptography.hazmat.primitives.asymmetric.rsa import RSAPrivateKey
@@ -45,6 +45,7 @@ from .auth import (
     AuthByWebBrowser,
 )
 from .auth.idtoken import AuthByIdToken
+from .backoff_policies import exponential_backoff
 from .bind_upload_agent import BindUploadError
 from .compat import IS_LINUX, IS_WINDOWS, quote, urlencode
 from .config_manager import CONFIG_MANAGER, _get_default_connection_params
@@ -82,6 +83,7 @@ from .errorcode import (
     ER_FAILED_PROCESSING_PYFORMAT,
     ER_FAILED_PROCESSING_QMARK,
     ER_FAILED_TO_CONNECT_TO_DB,
+    ER_INVALID_BACKOFF_POLICY,
     ER_INVALID_VALUE,
     ER_NO_ACCOUNT_NAME,
     ER_NO_NUMPY,
@@ -108,6 +110,7 @@ from .util_text import construct_hostname, parse_account, split_statements
 
 DEFAULT_CLIENT_PREFETCH_THREADS = 4
 MAX_CLIENT_PREFETCH_THREADS = 10
+DEFAULT_BACKOFF_POLICY = exponential_backoff()
 
 
 def DefaultConverterClass() -> type:
@@ -146,11 +149,13 @@ DEFAULT_CONFIGURATION: dict[str, tuple[Any, type | tuple[type, ...]]] = {
     "schema": (None, (type(None), str)),  # snowflake
     "role": (None, (type(None), str)),  # snowflake
     "session_id": (None, (type(None), str)),  # snowflake
-    "login_timeout": (120, int),  # login timeout
+    "login_timeout": (None, (type(None), int)),  # login timeout
     "network_timeout": (
         None,
         (type(None), int),
     ),  # network timeout (infinite by default)
+    "socket_timeout": (None, (type(None), int)),
+    "backoff_policy": (DEFAULT_BACKOFF_POLICY, Callable),
     "passcode_in_password": (False, bool),  # Snowflake MFA
     "passcode": (None, (type(None), str)),  # Snowflake MFA
     "private_key": (None, (type(None), str, RSAPrivateKey)),
@@ -189,9 +194,9 @@ DEFAULT_CONFIGURATION: dict[str, tuple[Any, type | tuple[type, ...]]] = {
     "client_store_temporary_credential": (False, bool),
     "client_request_mfa_token": (False, bool),
     "use_openssl_only": (
-        False,
+        True,
         bool,
-    ),  # only use openssl instead of python only crypto modules
+    ),  # ignored - python only crypto modules are no longer used
     # whether to convert Arrow number values to decimal instead of doubles
     "arrow_number_to_decimal": (False, bool),
     "enable_stage_s3_privatelink_for_us_east_1": (
@@ -276,8 +281,20 @@ class SnowflakeConnection:
         schema: Schema in use on Snowflake.
         warehouse: Warehouse to be used on Snowflake.
         role: Role in use on Snowflake.
-        login_timeout: Login timeout in seconds. Used while authenticating.
-        network_timeout: Network timeout. Used for general purpose.
+        login_timeout: Login timeout in seconds. Login requests will not be retried after this timeout expires.
+            Note that the login attempt may still take more than login_timeout seconds as an ongoing login request
+            cannot be canceled even upon login timeout expiry. The login timeout only prevents further retries.
+            If not specified, login_timeout is set to `snowflake.connector.auth.by_plugin.DEFAULT_AUTH_CLASS_TIMEOUT`.
+            Note that the number of retries on login requests is still limited by
+            `snowflake.connector.auth.by_plugin.DEFAULT_MAX_CON_RETRY_ATTEMPTS`.
+        network_timeout: Network timeout in seconds. Network requests besides login requests will not be retried
+            after this timeout expires. Overriden in cursor query execution if timeout is passed to cursor.execute.
+            Note that an operation may still take more than network_timeout seconds for the same reason as above.
+            If not specified, network_timeout is infinite.
+        socket_timeout: Socket timeout in seconds. Sets both socket connect and read timeout.
+        backoff_policy: Backoff policy to use for login and network requests. Must be a callable generator function.
+            Standard linear and exponential backoff implementations are included in `snowflake.connector.backoff_policies`
+            See the backoff_policies module for details and implementation examples.
         client_session_keep_alive_heartbeat_frequency: Heartbeat frequency to keep connection alive in seconds.
         client_prefetch_threads: Number of threads to download the result set.
         rest: Snowflake REST API object. Internal use only. Maybe removed in a later release.
@@ -287,7 +304,6 @@ class SnowflakeConnection:
         validate_default_parameters: Validate database, schema, role and warehouse used on Snowflake.
         is_pyformat: Whether the current argument binding is pyformat or format.
         consent_cache_id_token: Consented cache ID token.
-        use_openssl_only: Use OpenSSL instead of pure Python libraries for signature verification and encryption.
         enable_stage_s3_privatelink_for_us_east_1: when true, clients use regional s3 url to upload files.
         enable_connection_diag: when true, clients will generate a connectivity diagnostic report.
         connection_diag_log_path: path to location to create diag report with enable_connection_diag.
@@ -479,6 +495,14 @@ class SnowflakeConnection:
     def network_timeout(self) -> int | None:
         return int(self._network_timeout) if self._network_timeout is not None else None
 
+    @property
+    def socket_timeout(self) -> int | None:
+        return int(self._socket_timeout) if self._socket_timeout is not None else None
+
+    @property
+    def _backoff_generator(self) -> Iterator:
+        return self._backoff_policy()
+
     @property
     def client_session_keep_alive(self) -> bool | None:
         return self._client_session_keep_alive
@@ -574,7 +598,8 @@ class SnowflakeConnection:
 
     @property
     def use_openssl_only(self) -> bool:
-        return self._use_openssl_only
+        # Deprecated, kept for backwards compatibility
+        return True
 
     @property
     def arrow_number_to_decimal(self):
@@ -882,7 +907,11 @@ class SnowflakeConnection:
                 # TODO: add telemetry for custom auth
             self.auth_class = self.auth_class
         elif self._authenticator == DEFAULT_AUTHENTICATOR:
-            self.auth_class = AuthByDefault(password=self._password)
+            self.auth_class = AuthByDefault(
+                password=self._password,
+                timeout=self._login_timeout,
+                backoff_generator=self._backoff_generator,
+            )
         elif self._authenticator == EXTERNAL_BROWSER_AUTHENTICATOR:
             self._session_parameters[PARAMETER_CLIENT_STORE_TEMPORARY_CREDENTIAL] = (
                 self._client_store_temporary_credential if IS_LINUX else True
@@ -900,6 +929,8 @@ class SnowflakeConnection:
                     protocol=self._protocol,
                     host=self.host,
                     port=self.port,
+                    timeout=self._login_timeout,
+                    backoff_generator=self._backoff_generator,
                 )
             else:
                 self.auth_class = AuthByIdToken(
@@ -908,12 +939,22 @@ class SnowflakeConnection:
                     protocol=self._protocol,
                     host=self.host,
                     port=self.port,
+                    timeout=self._login_timeout,
+                    backoff_generator=self._backoff_generator,
                 )
 
         elif self._authenticator == KEY_PAIR_AUTHENTICATOR:
-            self.auth_class = AuthByKeyPair(private_key=self._private_key)
+            self.auth_class = AuthByKeyPair(
+                private_key=self._private_key,
+                timeout=self._login_timeout,
+                backoff_generator=self._backoff_generator,
+            )
         elif self._authenticator == OAUTH_AUTHENTICATOR:
-            self.auth_class = AuthByOAuth(oauth_token=self._token)
+            self.auth_class = AuthByOAuth(
+                oauth_token=self._token,
+                timeout=self._login_timeout,
+                backoff_generator=self._backoff_generator,
+            )
         elif self._authenticator == USR_PWD_MFA_AUTHENTICATOR:
             self._session_parameters[PARAMETER_CLIENT_REQUEST_MFA_TOKEN] = (
                 self._client_request_mfa_token if IS_LINUX else True
@@ -927,10 +968,16 @@ class SnowflakeConnection:
             self.auth_class = AuthByUsrPwdMfa(
                 password=self._password,
                 mfa_token=self.rest.mfa_token,
+                timeout=self._login_timeout,
+                backoff_generator=self._backoff_generator,
             )
         else:
             # okta URL, e.g., https://<account>.okta.com/
-            self.auth_class = AuthByOkta(application=self.application)
+            self.auth_class = AuthByOkta(
+                application=self.application,
+                timeout=self._login_timeout,
+                backoff_generator=self._backoff_generator,
+            )
 
         self.authenticate_with_retry(self.auth_class)
 
@@ -1099,6 +1146,19 @@ class SnowflakeConnection:
         if "." in self._account:
             self._account = parse_account(self._account)
 
+        if not isinstance(self._backoff_policy, Callable) or not isinstance(
+            self._backoff_policy(), Iterator
+        ):
+            Error.errorhandler_wrapper(
+                self,
+                None,
+                ProgrammingError,
+                {
+                    "msg": "Backoff policy must be a generator function",
+                    "errno": ER_INVALID_BACKOFF_POLICY,
+                },
+            )
+
         if self.ocsp_fail_open:
             logger.info(
                 "This connection is in OCSP Fail Open Mode. "
@@ -1117,19 +1177,6 @@ class SnowflakeConnection:
                 "CHECKED."
             )
 
-        if "SF_USE_OPENSSL_ONLY" not in os.environ:
-            logger.info("Setting use_openssl_only mode to %s", self.use_openssl_only)
-            os.environ["SF_USE_OPENSSL_ONLY"] = str(self.use_openssl_only)
-        elif (
-            os.environ.get("SF_USE_OPENSSL_ONLY", "False") == "True"
-        ) != self.use_openssl_only:
-            logger.warning(
-                "Mode use_openssl_only is already set to: %s, ignoring set request to: %s",
-                os.environ["SF_USE_OPENSSL_ONLY"],
-                self.use_openssl_only,
-            )
-            self._use_openssl_only = os.environ["SF_USE_OPENSSL_ONLY"] == "True"
-
     def cmd_query(
         self,
         sql: str,
@@ -1144,6 +1191,7 @@ class SnowflakeConnection:
         _no_results: bool = False,
         _update_current_object: bool = True,
         _no_retry: bool = False,
+        timeout: int | None = None,
     ) -> dict[str, Any]:
         """Executes a query with a sequence counter."""
         logger.debug("_cmd_query")
@@ -1189,6 +1237,7 @@ class SnowflakeConnection:
             _no_results=_no_results,
             _include_retry_params=True,
             _no_retry=_no_retry,
+            timeout=timeout,
         )
 
         if ret is None:
@@ -1242,6 +1291,8 @@ class SnowflakeConnection:
         )
 
         auth = Auth(self.rest)
+        # record start time for computing timeout
+        auth_instance._retry_ctx.set_start_time()
         try:
             auth.authenticate(
                 auth_instance=auth_instance,
@@ -1262,7 +1313,6 @@ class SnowflakeConnection:
                 "Operational Error raised at authentication"
                 f"for authenticator: {type(auth_instance).__name__}"
             )
-
             while True:
                 try:
                     auth_instance.handle_timeout(
